*! version 1.0.0 October 18, 2016 @ 10:16:26
*! computes little-omega values from calinski-harabasz pseudo-F's
program define chomega
version 14.2

   // this really should be done with the matrix option and mata
   // only works on first contiguous block of groups
   if `"`r(rule)'"'!="calinski" {
      display as error "last command not cluster stop ..., rule(calinski)"
      exit 301
      }
   
   syntax [, format(str)]
   if `"`format'"'=="" {
      local format "%7.3f"
      }
   else {
      capture local tmp : display `format' 1
		if _rc {
			di as err `"invalid %fmt in format(): `format'"'
			exit 120
         }
      }

   // finding the lowest-numbered group
   local min = 2
   while missing(r(calinski_`min')) {
      local ++min
      }

   // finding the highest-numbered sequential group
   //   this is really just so that the displayed results align
   local max = `min'
   while !missing(r(calinski_`max')) {
      local ++max
      }
   local --max

   // checking to see if the first set of consecutive groups has at least
   //   three groups in it
   if `max'<`min'+2 {
      display as result "Not enough consecutive groups to compute little omega; you must have at least 3"
      exit
      }
   local spacemax = ceil(log10(`max'+0.5))
   tempname omega minomega
   scalar `minomega'=.
   local mink=.
   forvalues k=`=`min'+1'/`=`max'-1' {
      // computing omega separately in case error checks might be needed
      scalar `omega'= r(calinski_`=`k'+1')-2*r(calinski_`k')+r(calinski_`=`k'-1')
      if `omega'<`minomega' {
         scalar `minomega'=`omega'
         local mink=`k'
         }
      local spacer = (`spacemax'-ceil(log10(`k'+0.5)))*" "
      display as text ("omega_`k' " + `"`spacer'"' + "is ") `format' `omega'
      local ++k
      }
   display as result "Minimum value of omega: " `format' `minomega' " at `mink' clusters"
 
end
